body {
    background: #121212;  /* Dark mode background */
    color: #f0f0f0;        /* Light text color */
    display: flex;
    justify-content: center;
    align-items: center;
    height: 100vh;
    margin: 0;
}

#main {
    width: 400px;
    max-width: 800px;        /* Limit the width of the main content */
    background: #333;        /* Dark background for main content */
    border-radius: 20px;
    box-shadow: 0 10px 30px rgba(0, 0, 0, 0.5);  /* Floating effect */
    padding: 20px;
    display: flex;
    flex-direction: column;  /* Stack elements vertically */
    align-items: center;     /* Center content horizontally */
    gap: 20px;               /* Space between elements */
}

h1 {
    font-size: 24px;
    font-weight: bold;
    margin: 0;
    text-align: center;
}

#paragraphs-wrapper {
    width: 100%;             /* Make paragraphs take full width */
    display: flex;
    flex-direction: column;  /* Stack paragraphs vertically */
    gap: 10px;               /* Space between paragraphs */
}

#paragraphs-wrapper p {
    margin: 0;
    font-size: 16px;
    line-height: 1.6;
    text-align: left;        /* Align text to the left */
}

button {
    padding: 10px 20px;
    font-size: 16px;
    background-color: #444;
    color: white;
    border: none;
    border-radius: 5px;
    cursor: pointer;
    transition: background-color 0.3s ease;
}

button:hover {
    background-color: #555;  /* Darken button on hover */
}

button:disabled {
    background-color: #665;
    cursor: not-allowed;
}
